@@ -27,30 +27,34 @@
 
 
 #include "main/imports.h"
-#include "main/mfeatures.h"
 #include "main/mtypes.h"
 #include "main/macros.h"
 #include "main/bufferobj.h"
 
+#include "brw_context.h"
 #include "intel_blit.h"
 #include "intel_buffer_objects.h"
 #include "intel_batchbuffer.h"
-#include "intel_context.h"
 #include "intel_fbo.h"
 #include "intel_mipmap_tree.h"
 #include "intel_regions.h"
 
+#include "brw_context.h"
+
 static GLboolean
-intel_bufferobj_unmap(struct gl_context * ctx,
-                      GLenum target, struct gl_buffer_object *obj);
+intel_bufferobj_unmap(struct gl_context * ctx, struct gl_buffer_object *obj);
 
 /** Allocates a new drm_intel_bo to store the data for the buffer object. */
 static void
-intel_bufferobj_alloc_buffer(struct intel_context *intel,
+intel_bufferobj_alloc_buffer(struct brw_context *brw,
 			     struct intel_buffer_object *intel_obj)
 {
-   intel_obj->buffer = drm_intel_bo_alloc(intel->bufmgr, "bufferobj",
+   intel_obj->buffer = drm_intel_bo_alloc(brw->bufmgr, "bufferobj",
 					  intel_obj->Base.Size, 64);
+
+   /* the buffer might be bound as a uniform buffer, need to update it
+    */
+   brw->state.dirty.brw |= BRW_NEW_UNIFORM_BUFFER;
 }
 
 static void
@@ -59,7 +63,6 @@ release_buffer(struct intel_buffer_object *intel_obj)
    drm_intel_bo_unreference(intel_obj->buffer);
    intel_obj->buffer = NULL;
    intel_obj->offset = 0;
-   intel_obj->source = 0;
 }
 
 /**
@@ -73,38 +76,13 @@ intel_bufferobj_alloc(struct gl_context * ctx, GLuint name, GLenum target)
 {
    struct intel_buffer_object *obj = CALLOC_STRUCT(intel_buffer_object);
 
-   _mesa_initialize_buffer_object(&obj->Base, name, target);
+   _mesa_initialize_buffer_object(ctx, &obj->Base, name, target);
 
    obj->buffer = NULL;
 
    return &obj->Base;
 }
 
-/* Break the COW tie to the region.  The region gets to keep the data.
- */
-void
-intel_bufferobj_release_region(struct intel_context *intel,
-                               struct intel_buffer_object *intel_obj)
-{
-   assert(intel_obj->region->buffer == intel_obj->buffer);
-   intel_obj->region->pbo = NULL;
-   intel_obj->region = NULL;
-
-   release_buffer(intel_obj);
-}
-
-/* Break the COW tie to the region.  Both the pbo and the region end
- * up with a copy of the data.
- */
-void
-intel_bufferobj_cow(struct intel_context *intel,
-                    struct intel_buffer_object *intel_obj)
-{
-   assert(intel_obj->region);
-   intel_region_cow(intel, intel_obj->region);
-}
-
-
 /**
  * Deallocate/free a vertex/pixel buffer object.
  * Called via glDeleteBuffersARB().
@@ -112,7 +90,6 @@ intel_bufferobj_cow(struct intel_context *intel,
 static void
 intel_bufferobj_free(struct gl_context * ctx, struct gl_buffer_object *obj)
 {
-   struct intel_context *intel = intel_context(ctx);
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
 
    assert(intel_obj);
@@ -122,12 +99,7 @@ intel_bufferobj_free(struct gl_context * ctx, struct gl_buffer_object *obj)
     * (though it does if you call glDeleteBuffers)
     */
    if (obj->Pointer)
-      intel_bufferobj_unmap(ctx, 0, obj);
-
-   free(intel_obj->sys_buffer);
-   if (intel_obj->region) {
-      intel_bufferobj_release_region(intel, intel_obj);
-   }
+      intel_bufferobj_unmap(ctx, obj);
 
    drm_intel_bo_unreference(intel_obj->buffer);
    free(intel_obj);
@@ -140,7 +112,7 @@ intel_bufferobj_free(struct gl_context * ctx, struct gl_buffer_object *obj)
  * previously stored in the buffer object is lost.  If data is NULL,
  * memory will be allocated, but no copy will occur.
  * Called via ctx->Driver.BufferData().
- * \return GL_TRUE for success, GL_FALSE if out of memory
+ * \return true for success, false if out of memory
  */
 static GLboolean
 intel_bufferobj_data(struct gl_context * ctx,
@@ -149,49 +121,31 @@ intel_bufferobj_data(struct gl_context * ctx,
                      const GLvoid * data,
                      GLenum usage, struct gl_buffer_object *obj)
 {
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
 
+   /* Part of the ABI, but this function doesn't use it.
+    */
+   (void) target;
+
    intel_obj->Base.Size = size;
    intel_obj->Base.Usage = usage;
 
    assert(!obj->Pointer); /* Mesa should have unmapped it */
 
-   if (intel_obj->region)
-      intel_bufferobj_release_region(intel, intel_obj);
-
    if (intel_obj->buffer != NULL)
       release_buffer(intel_obj);
 
-   free(intel_obj->sys_buffer);
-   intel_obj->sys_buffer = NULL;
-
    if (size != 0) {
-      if (usage == GL_DYNAMIC_DRAW
-#ifdef I915
-	  /* On pre-965, stick VBOs in system memory, as we're always doing
-	   * swtnl with their contents anyway.
-	   */
-	  || target == GL_ARRAY_BUFFER || target == GL_ELEMENT_ARRAY_BUFFER
-#endif
-	 )
-      {
-	 intel_obj->sys_buffer = malloc(size);
-	 if (intel_obj->sys_buffer != NULL) {
-	    if (data != NULL)
-	       memcpy(intel_obj->sys_buffer, data, size);
-	    return GL_TRUE;
-	 }
-      }
-      intel_bufferobj_alloc_buffer(intel, intel_obj);
+      intel_bufferobj_alloc_buffer(brw, intel_obj);
       if (!intel_obj->buffer)
-         return GL_FALSE;
+         return false;
 
       if (data != NULL)
 	 drm_intel_bo_subdata(intel_obj->buffer, 0, size, data);
    }
 
-   return GL_TRUE;
+   return true;
 }
 
 
@@ -203,12 +157,11 @@ intel_bufferobj_data(struct gl_context * ctx,
  */
 static void
 intel_bufferobj_subdata(struct gl_context * ctx,
-                        GLenum target,
                         GLintptrARB offset,
                         GLsizeiptrARB size,
                         const GLvoid * data, struct gl_buffer_object *obj)
 {
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
    bool busy;
 
@@ -217,55 +170,33 @@ intel_bufferobj_subdata(struct gl_context * ctx,
 
    assert(intel_obj);
 
-   if (intel_obj->region)
-      intel_bufferobj_cow(intel, intel_obj);
-
-   /* If we have a single copy in system memory, update that */
-   if (intel_obj->sys_buffer) {
-      if (intel_obj->source)
-	 release_buffer(intel_obj);
-
-      if (intel_obj->buffer == NULL) {
-	 memcpy((char *)intel_obj->sys_buffer + offset, data, size);
-	 return;
-      }
-
-      free(intel_obj->sys_buffer);
-      intel_obj->sys_buffer = NULL;
-   }
-
-   /* Otherwise we need to update the copy in video memory. */
    busy =
       drm_intel_bo_busy(intel_obj->buffer) ||
-      drm_intel_bo_references(intel->batch.bo, intel_obj->buffer);
+      drm_intel_bo_references(brw->batch.bo, intel_obj->buffer);
 
-   /* replace the current busy bo with fresh data */
-   if (busy && size == intel_obj->Base.Size) {
-      drm_intel_bo_unreference(intel_obj->buffer);
-      intel_bufferobj_alloc_buffer(intel, intel_obj);
-      drm_intel_bo_subdata(intel_obj->buffer, 0, size, data);
-   } else if (intel->gen < 6) {
-      if (busy) {
-	 drm_intel_bo *temp_bo;
-
-	 temp_bo = drm_intel_bo_alloc(intel->bufmgr, "subdata temp", size, 64);
+   if (busy) {
+      if (size == intel_obj->Base.Size) {
+	 /* Replace the current busy bo with fresh data. */
+	 drm_intel_bo_unreference(intel_obj->buffer);
+	 intel_bufferobj_alloc_buffer(brw, intel_obj);
+	 drm_intel_bo_subdata(intel_obj->buffer, 0, size, data);
+      } else {
+         perf_debug("Using a blit copy to avoid stalling on %ldb "
+                    "glBufferSubData() to a busy buffer object.\n",
+                    (long)size);
+	 drm_intel_bo *temp_bo =
+	    drm_intel_bo_alloc(brw->bufmgr, "subdata temp", size, 64);
 
 	 drm_intel_bo_subdata(temp_bo, 0, size, data);
 
-	 intel_emit_linear_blit(intel,
+	 intel_emit_linear_blit(brw,
 				intel_obj->buffer, offset,
 				temp_bo, 0,
 				size);
 
 	 drm_intel_bo_unreference(temp_bo);
-      } else {
-	 drm_intel_bo_subdata(intel_obj->buffer, offset, size, data);
       }
    } else {
-      /* Can't use the blit to modify the buffer in the middle of batch. */
-      if (drm_intel_bo_references(intel->batch.bo, intel_obj->buffer)) {
-	 intel_batchbuffer_flush(intel);
-      }
       drm_intel_bo_subdata(intel_obj->buffer, offset, size, data);
    }
 }
@@ -276,87 +207,24 @@ intel_bufferobj_subdata(struct gl_context * ctx,
  */
 static void
 intel_bufferobj_get_subdata(struct gl_context * ctx,
-                            GLenum target,
                             GLintptrARB offset,
                             GLsizeiptrARB size,
                             GLvoid * data, struct gl_buffer_object *obj)
 {
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
 
    assert(intel_obj);
-   if (intel_obj->sys_buffer)
-      memcpy(data, (char *)intel_obj->sys_buffer + offset, size);
-   else {
-      if (drm_intel_bo_references(intel->batch.bo, intel_obj->buffer)) {
-	 intel_batchbuffer_flush(intel);
-      }
-      drm_intel_bo_get_subdata(intel_obj->buffer, offset, size, data);
+   if (drm_intel_bo_references(brw->batch.bo, intel_obj->buffer)) {
+      intel_batchbuffer_flush(brw);
    }
+   drm_intel_bo_get_subdata(intel_obj->buffer, offset, size, data);
 }
 
 
 
 /**
- * Called via glMapBufferARB().
- */
-static void *
-intel_bufferobj_map(struct gl_context * ctx,
-                    GLenum target,
-                    GLenum access, struct gl_buffer_object *obj)
-{
-   struct intel_context *intel = intel_context(ctx);
-   struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
-   GLboolean read_only = (access == GL_READ_ONLY_ARB);
-   GLboolean write_only = (access == GL_WRITE_ONLY_ARB);
-
-   assert(intel_obj);
-
-   if (intel_obj->sys_buffer) {
-      if (!read_only && intel_obj->source) {
-	 release_buffer(intel_obj);
-      }
-
-      if (!intel_obj->buffer || intel_obj->source) {
-	 obj->Pointer = intel_obj->sys_buffer;
-	 obj->Length = obj->Size;
-	 obj->Offset = 0;
-	 return obj->Pointer;
-      }
-
-      free(intel_obj->sys_buffer);
-      intel_obj->sys_buffer = NULL;
-   }
-
-   /* Flush any existing batchbuffer that might reference this data. */
-   if (drm_intel_bo_references(intel->batch.bo, intel_obj->buffer))
-      intel_flush(ctx);
-
-   if (intel_obj->region)
-      intel_bufferobj_cow(intel, intel_obj);
-
-   if (intel_obj->buffer == NULL) {
-      obj->Pointer = NULL;
-      return NULL;
-   }
-
-   if (write_only) {
-      drm_intel_gem_bo_map_gtt(intel_obj->buffer);
-      intel_obj->mapped_gtt = GL_TRUE;
-   } else {
-      drm_intel_bo_map(intel_obj->buffer, !read_only);
-      intel_obj->mapped_gtt = GL_FALSE;
-   }
-
-   obj->Pointer = intel_obj->buffer->virtual;
-   obj->Length = obj->Size;
-   obj->Offset = 0;
-
-   return obj->Pointer;
-}
-
-/**
- * Called via glMapBufferRange().
+ * Called via glMapBufferRange and glMapBuffer
  *
  * The goal of this extension is to allow apps to accumulate their rendering
  * at the same time as they accumulate their buffer object.  Without it,
@@ -373,12 +241,11 @@ intel_bufferobj_map(struct gl_context * ctx,
  */
 static void *
 intel_bufferobj_map_range(struct gl_context * ctx,
-			  GLenum target, GLintptr offset, GLsizeiptr length,
+			  GLintptr offset, GLsizeiptr length,
 			  GLbitfield access, struct gl_buffer_object *obj)
 {
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
-   GLboolean read_only = (access == GL_READ_ONLY_ARB);
 
    assert(intel_obj);
 
@@ -389,43 +256,34 @@ intel_bufferobj_map_range(struct gl_context * ctx,
    obj->Length = length;
    obj->AccessFlags = access;
 
-   if (intel_obj->sys_buffer) {
-      if (!read_only && intel_obj->source)
-	 release_buffer(intel_obj);
-
-      if (!intel_obj->buffer || intel_obj->source) {
-	 obj->Pointer = intel_obj->sys_buffer + offset;
-	 return obj->Pointer;
-      }
-
-      free(intel_obj->sys_buffer);
-      intel_obj->sys_buffer = NULL;
-   }
-
-   if (intel_obj->region)
-      intel_bufferobj_cow(intel, intel_obj);
-
-   /* If the mapping is synchronized with other GL operations, flush
-    * the batchbuffer so that GEM knows about the buffer access for later
-    * syncing.
-    */
-   if (!(access & GL_MAP_UNSYNCHRONIZED_BIT) &&
-       drm_intel_bo_references(intel->batch.bo, intel_obj->buffer))
-      intel_flush(ctx);
-
    if (intel_obj->buffer == NULL) {
       obj->Pointer = NULL;
       return NULL;
    }
 
-   /* If the user doesn't care about existing buffer contents and mapping
-    * would cause us to block, then throw out the old buffer.
+   /* If the access is synchronized (like a normal buffer mapping), then get
+    * things flushed out so the later mapping syncs appropriately through GEM.
+    * If the user doesn't care about existing buffer contents and mapping would
+    * cause us to block, then throw out the old buffer.
+    *
+    * If they set INVALIDATE_BUFFER, we can pitch the current contents to
+    * achieve the required synchronization.
     */
-   if (!(access & GL_MAP_UNSYNCHRONIZED_BIT) &&
-       (access & GL_MAP_INVALIDATE_BUFFER_BIT) &&
-       drm_intel_bo_busy(intel_obj->buffer)) {
-      drm_intel_bo_unreference(intel_obj->buffer);
-      intel_bufferobj_alloc_buffer(intel, intel_obj);
+   if (!(access & GL_MAP_UNSYNCHRONIZED_BIT)) {
+      if (drm_intel_bo_references(brw->batch.bo, intel_obj->buffer)) {
+	 if (access & GL_MAP_INVALIDATE_BUFFER_BIT) {
+	    drm_intel_bo_unreference(intel_obj->buffer);
+	    intel_bufferobj_alloc_buffer(brw, intel_obj);
+	 } else {
+            perf_debug("Stalling on the GPU for mapping a busy buffer "
+                       "object\n");
+	    intel_flush(ctx);
+	 }
+      } else if (drm_intel_bo_busy(intel_obj->buffer) &&
+		 (access & GL_MAP_INVALIDATE_BUFFER_BIT)) {
+	 drm_intel_bo_unreference(intel_obj->buffer);
+	 intel_bufferobj_alloc_buffer(brw, intel_obj);
+      }
    }
 
    /* If the user is mapping a range of an active buffer object but
@@ -439,28 +297,26 @@ intel_bufferobj_map_range(struct gl_context * ctx,
 	 intel_obj->range_map_buffer = malloc(length);
 	 obj->Pointer = intel_obj->range_map_buffer;
       } else {
-	 intel_obj->range_map_bo = drm_intel_bo_alloc(intel->bufmgr,
+	 intel_obj->range_map_bo = drm_intel_bo_alloc(brw->bufmgr,
 						      "range map",
 						      length, 64);
 	 if (!(access & GL_MAP_READ_BIT)) {
 	    drm_intel_gem_bo_map_gtt(intel_obj->range_map_bo);
-	    intel_obj->mapped_gtt = GL_TRUE;
 	 } else {
 	    drm_intel_bo_map(intel_obj->range_map_bo,
 			     (access & GL_MAP_WRITE_BIT) != 0);
-	    intel_obj->mapped_gtt = GL_FALSE;
 	 }
 	 obj->Pointer = intel_obj->range_map_bo->virtual;
       }
       return obj->Pointer;
    }
 
-   if (!(access & GL_MAP_READ_BIT)) {
+   if (access & GL_MAP_UNSYNCHRONIZED_BIT)
+      drm_intel_gem_bo_map_unsynchronized(intel_obj->buffer);
+   else if (!(access & GL_MAP_READ_BIT)) {
       drm_intel_gem_bo_map_gtt(intel_obj->buffer);
-      intel_obj->mapped_gtt = GL_TRUE;
    } else {
       drm_intel_bo_map(intel_obj->buffer, (access & GL_MAP_WRITE_BIT) != 0);
-      intel_obj->mapped_gtt = GL_FALSE;
    }
 
    obj->Pointer = intel_obj->buffer->virtual + offset;
@@ -473,11 +329,11 @@ intel_bufferobj_map_range(struct gl_context * ctx,
  * would defeat the point.
  */
 static void
-intel_bufferobj_flush_mapped_range(struct gl_context *ctx, GLenum target,
+intel_bufferobj_flush_mapped_range(struct gl_context *ctx,
 				   GLintptr offset, GLsizeiptr length,
 				   struct gl_buffer_object *obj)
 {
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
    drm_intel_bo *temp_bo;
 
@@ -490,11 +346,11 @@ intel_bufferobj_flush_mapped_range(struct gl_context *ctx, GLenum target,
    if (length == 0)
       return;
 
-   temp_bo = drm_intel_bo_alloc(intel->bufmgr, "range map flush", length, 64);
+   temp_bo = drm_intel_bo_alloc(brw->bufmgr, "range map flush", length, 64);
 
    drm_intel_bo_subdata(temp_bo, 0, length, intel_obj->range_map_buffer);
 
-   intel_emit_linear_blit(intel,
+   intel_emit_linear_blit(brw,
 			  intel_obj->buffer, obj->Offset + offset,
 			  temp_bo, 0,
 			  length);
@@ -507,33 +363,26 @@ intel_bufferobj_flush_mapped_range(struct gl_context *ctx, GLenum target,
  * Called via glUnmapBuffer().
  */
 static GLboolean
-intel_bufferobj_unmap(struct gl_context * ctx,
-                      GLenum target, struct gl_buffer_object *obj)
+intel_bufferobj_unmap(struct gl_context * ctx, struct gl_buffer_object *obj)
 {
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
    struct intel_buffer_object *intel_obj = intel_buffer_object(obj);
 
    assert(intel_obj);
    assert(obj->Pointer);
-   if (intel_obj->sys_buffer != NULL) {
-      /* always keep the mapping around. */
-   } else if (intel_obj->range_map_buffer != NULL) {
+   if (intel_obj->range_map_buffer != NULL) {
       /* Since we've emitted some blits to buffers that will (likely) be used
        * in rendering operations in other cache domains in this batch, emit a
        * flush.  Once again, we wish for a domain tracker in libdrm to cover
        * usage inside of a batchbuffer.
        */
-      intel_batchbuffer_emit_mi_flush(intel);
+      intel_batchbuffer_emit_mi_flush(brw);
       free(intel_obj->range_map_buffer);
       intel_obj->range_map_buffer = NULL;
    } else if (intel_obj->range_map_bo != NULL) {
-      if (intel_obj->mapped_gtt) {
-	 drm_intel_gem_bo_unmap_gtt(intel_obj->range_map_bo);
-      } else {
-	 drm_intel_bo_unmap(intel_obj->range_map_bo);
-      }
+      drm_intel_bo_unmap(intel_obj->range_map_bo);
 
-      intel_emit_linear_blit(intel,
+      intel_emit_linear_blit(brw,
 			     intel_obj->buffer, obj->Offset,
 			     intel_obj->range_map_bo, 0,
 			     obj->Length);
@@ -543,51 +392,27 @@ intel_bufferobj_unmap(struct gl_context * ctx,
        * flush.  Once again, we wish for a domain tracker in libdrm to cover
        * usage inside of a batchbuffer.
        */
-      intel_batchbuffer_emit_mi_flush(intel);
+      intel_batchbuffer_emit_mi_flush(brw);
 
       drm_intel_bo_unreference(intel_obj->range_map_bo);
       intel_obj->range_map_bo = NULL;
    } else if (intel_obj->buffer != NULL) {
-      if (intel_obj->mapped_gtt) {
-	 drm_intel_gem_bo_unmap_gtt(intel_obj->buffer);
-      } else {
-	 drm_intel_bo_unmap(intel_obj->buffer);
-      }
+      drm_intel_bo_unmap(intel_obj->buffer);
    }
    obj->Pointer = NULL;
    obj->Offset = 0;
    obj->Length = 0;
 
-   return GL_TRUE;
+   return true;
 }
 
 drm_intel_bo *
-intel_bufferobj_buffer(struct intel_context *intel,
+intel_bufferobj_buffer(struct brw_context *brw,
                        struct intel_buffer_object *intel_obj,
 		       GLuint flag)
 {
-   if (intel_obj->region) {
-      if (flag == INTEL_WRITE_PART)
-         intel_bufferobj_cow(intel, intel_obj);
-      else if (flag == INTEL_WRITE_FULL) {
-         intel_bufferobj_release_region(intel, intel_obj);
-	 intel_bufferobj_alloc_buffer(intel, intel_obj);
-      }
-   }
-
-   if (intel_obj->source)
-      release_buffer(intel_obj);
-
-   if (intel_obj->buffer == NULL) {
-      intel_bufferobj_alloc_buffer(intel, intel_obj);
-      drm_intel_bo_subdata(intel_obj->buffer,
-			   0, intel_obj->Base.Size,
-			   intel_obj->sys_buffer);
-
-      free(intel_obj->sys_buffer);
-      intel_obj->sys_buffer = NULL;
-      intel_obj->offset = 0;
-   }
+   if (intel_obj->buffer == NULL)
+      intel_bufferobj_alloc_buffer(brw, intel_obj);
 
    return intel_obj->buffer;
 }
@@ -595,148 +420,141 @@ intel_bufferobj_buffer(struct intel_context *intel,
 #define INTEL_UPLOAD_SIZE (64*1024)
 
 void
-intel_upload_finish(struct intel_context *intel)
+intel_upload_finish(struct brw_context *brw)
 {
-   if (!intel->upload.bo)
+   if (!brw->upload.bo)
 	   return;
 
-   if (intel->upload.buffer_len) {
-	   drm_intel_bo_subdata(intel->upload.bo,
-				intel->upload.buffer_offset,
-				intel->upload.buffer_len,
-				intel->upload.buffer);
-	   intel->upload.buffer_len = 0;
+   if (brw->upload.buffer_len) {
+	   drm_intel_bo_subdata(brw->upload.bo,
+				brw->upload.buffer_offset,
+				brw->upload.buffer_len,
+				brw->upload.buffer);
+	   brw->upload.buffer_len = 0;
    }
 
-   drm_intel_bo_unreference(intel->upload.bo);
-   intel->upload.bo = NULL;
+   drm_intel_bo_unreference(brw->upload.bo);
+   brw->upload.bo = NULL;
 }
 
-static void wrap_buffers(struct intel_context *intel, GLuint size)
+static void wrap_buffers(struct brw_context *brw, GLuint size)
 {
-   intel_upload_finish(intel);
+   intel_upload_finish(brw);
 
    if (size < INTEL_UPLOAD_SIZE)
       size = INTEL_UPLOAD_SIZE;
 
-   intel->upload.bo = drm_intel_bo_alloc(intel->bufmgr, "upload", size, 0);
-   intel->upload.offset = 0;
+   brw->upload.bo = drm_intel_bo_alloc(brw->bufmgr, "upload", size, 0);
+   brw->upload.offset = 0;
 }
 
-void intel_upload_data(struct intel_context *intel,
+void intel_upload_data(struct brw_context *brw,
 		       const void *ptr, GLuint size, GLuint align,
 		       drm_intel_bo **return_bo,
 		       GLuint *return_offset)
 {
    GLuint base, delta;
 
-   base = (intel->upload.offset + align - 1) / align * align;
-   if (intel->upload.bo == NULL || base + size > intel->upload.bo->size) {
-      wrap_buffers(intel, size);
+   base = (brw->upload.offset + align - 1) / align * align;
+   if (brw->upload.bo == NULL || base + size > brw->upload.bo->size) {
+      wrap_buffers(brw, size);
       base = 0;
    }
 
-   drm_intel_bo_reference(intel->upload.bo);
-   *return_bo = intel->upload.bo;
+   drm_intel_bo_reference(brw->upload.bo);
+   *return_bo = brw->upload.bo;
    *return_offset = base;
 
-   delta = base - intel->upload.offset;
-   if (intel->upload.buffer_len &&
-       intel->upload.buffer_len + delta + size > sizeof(intel->upload.buffer))
+   delta = base - brw->upload.offset;
+   if (brw->upload.buffer_len &&
+       brw->upload.buffer_len + delta + size > sizeof(brw->upload.buffer))
    {
-      drm_intel_bo_subdata(intel->upload.bo,
-			   intel->upload.buffer_offset,
-			   intel->upload.buffer_len,
-			   intel->upload.buffer);
-      intel->upload.buffer_len = 0;
+      drm_intel_bo_subdata(brw->upload.bo,
+			   brw->upload.buffer_offset,
+			   brw->upload.buffer_len,
+			   brw->upload.buffer);
+      brw->upload.buffer_len = 0;
    }
 
-   if (size < sizeof(intel->upload.buffer))
+   if (size < sizeof(brw->upload.buffer))
    {
-      if (intel->upload.buffer_len == 0)
-	 intel->upload.buffer_offset = base;
+      if (brw->upload.buffer_len == 0)
+	 brw->upload.buffer_offset = base;
       else
-	 intel->upload.buffer_len += delta;
+	 brw->upload.buffer_len += delta;
 
-      memcpy(intel->upload.buffer + intel->upload.buffer_len, ptr, size);
-      intel->upload.buffer_len += size;
+      memcpy(brw->upload.buffer + brw->upload.buffer_len, ptr, size);
+      brw->upload.buffer_len += size;
    }
    else
    {
-      drm_intel_bo_subdata(intel->upload.bo, base, size, ptr);
+      drm_intel_bo_subdata(brw->upload.bo, base, size, ptr);
    }
 
-   intel->upload.offset = base + size;
+   brw->upload.offset = base + size;
 }
 
-void *intel_upload_map(struct intel_context *intel, GLuint size, GLuint align)
+void *intel_upload_map(struct brw_context *brw, GLuint size, GLuint align)
 {
    GLuint base, delta;
    char *ptr;
 
-   base = (intel->upload.offset + align - 1) / align * align;
-   if (intel->upload.bo == NULL || base + size > intel->upload.bo->size) {
-      wrap_buffers(intel, size);
+   base = (brw->upload.offset + align - 1) / align * align;
+   if (brw->upload.bo == NULL || base + size > brw->upload.bo->size) {
+      wrap_buffers(brw, size);
       base = 0;
    }
 
-   delta = base - intel->upload.offset;
-   if (intel->upload.buffer_len &&
-       intel->upload.buffer_len + delta + size > sizeof(intel->upload.buffer))
+   delta = base - brw->upload.offset;
+   if (brw->upload.buffer_len &&
+       brw->upload.buffer_len + delta + size > sizeof(brw->upload.buffer))
    {
-      drm_intel_bo_subdata(intel->upload.bo,
-			   intel->upload.buffer_offset,
-			   intel->upload.buffer_len,
-			   intel->upload.buffer);
-      intel->upload.buffer_len = 0;
+      drm_intel_bo_subdata(brw->upload.bo,
+			   brw->upload.buffer_offset,
+			   brw->upload.buffer_len,
+			   brw->upload.buffer);
+      brw->upload.buffer_len = 0;
    }
 
-   if (size <= sizeof(intel->upload.buffer)) {
-      if (intel->upload.buffer_len == 0)
-	 intel->upload.buffer_offset = base;
+   if (size <= sizeof(brw->upload.buffer)) {
+      if (brw->upload.buffer_len == 0)
+	 brw->upload.buffer_offset = base;
       else
-	 intel->upload.buffer_len += delta;
+	 brw->upload.buffer_len += delta;
 
-      ptr = intel->upload.buffer + intel->upload.buffer_len;
-      intel->upload.buffer_len += size;
+      ptr = brw->upload.buffer + brw->upload.buffer_len;
+      brw->upload.buffer_len += size;
    } else
       ptr = malloc(size);
 
    return ptr;
 }
 
-void intel_upload_unmap(struct intel_context *intel,
+void intel_upload_unmap(struct brw_context *brw,
 			const void *ptr, GLuint size, GLuint align,
 			drm_intel_bo **return_bo,
 			GLuint *return_offset)
 {
    GLuint base;
 
-   base = (intel->upload.offset + align - 1) / align * align;
-   if (size > sizeof(intel->upload.buffer)) {
-      drm_intel_bo_subdata(intel->upload.bo, base, size, ptr);
+   base = (brw->upload.offset + align - 1) / align * align;
+   if (size > sizeof(brw->upload.buffer)) {
+      drm_intel_bo_subdata(brw->upload.bo, base, size, ptr);
       free((void*)ptr);
    }
 
-   drm_intel_bo_reference(intel->upload.bo);
-   *return_bo = intel->upload.bo;
+   drm_intel_bo_reference(brw->upload.bo);
+   *return_bo = brw->upload.bo;
    *return_offset = base;
 
-   intel->upload.offset = base + size;
+   brw->upload.offset = base + size;
 }
 
 drm_intel_bo *
-intel_bufferobj_source(struct intel_context *intel,
+intel_bufferobj_source(struct brw_context *brw,
                        struct intel_buffer_object *intel_obj,
 		       GLuint align, GLuint *offset)
 {
-   if (intel_obj->buffer == NULL) {
-      intel_upload_data(intel,
-			intel_obj->sys_buffer, intel_obj->Base.Size, align,
-			&intel_obj->buffer, &intel_obj->offset);
-      intel_obj->source = 1;
-   }
-
    *offset = intel_obj->offset;
    return intel_obj->buffer;
 }
@@ -748,7 +566,7 @@ intel_bufferobj_copy_subdata(struct gl_context *ctx,
 			     GLintptr read_offset, GLintptr write_offset,
 			     GLsizeiptr size)
 {
-   struct intel_context *intel = intel_context(ctx);
+   struct brw_context *brw = brw_context(ctx);
    struct intel_buffer_object *intel_src = intel_buffer_object(src);
    struct intel_buffer_object *intel_dst = intel_buffer_object(dst);
    drm_intel_bo *src_bo, *dst_bo;
@@ -757,39 +575,10 @@ intel_bufferobj_copy_subdata(struct gl_context *ctx,
    if (size == 0)
       return;
 
-   /* If we're in system memory, just map and memcpy. */
-   if (intel_src->sys_buffer || intel_dst->sys_buffer || intel->gen >= 6) {
-      /* The same buffer may be used, but note that regions copied may
-       * not overlap.
-       */
-      if (src == dst) {
-	 char *ptr = intel_bufferobj_map(ctx, GL_COPY_WRITE_BUFFER,
-					 GL_READ_WRITE, dst);
-	 memmove(ptr + write_offset, ptr + read_offset, size);
-	 intel_bufferobj_unmap(ctx, GL_COPY_WRITE_BUFFER, dst);
-      } else {
-	 const char *src_ptr;
-	 char *dst_ptr;
+   dst_bo = intel_bufferobj_buffer(brw, intel_dst, INTEL_WRITE_PART);
+   src_bo = intel_bufferobj_source(brw, intel_src, 64, &src_offset);
 
-	 src_ptr =  intel_bufferobj_map(ctx, GL_COPY_READ_BUFFER,
-					GL_READ_ONLY, src);
-	 dst_ptr =  intel_bufferobj_map(ctx, GL_COPY_WRITE_BUFFER,
-					GL_WRITE_ONLY, dst);
-
-	 memcpy(dst_ptr + write_offset, src_ptr + read_offset, size);
-
-	 intel_bufferobj_unmap(ctx, GL_COPY_READ_BUFFER, src);
-	 intel_bufferobj_unmap(ctx, GL_COPY_WRITE_BUFFER, dst);
-      }
-      return;
-   }
-
-   /* Otherwise, we have real BOs, so blit them. */
-
-   dst_bo = intel_bufferobj_buffer(intel, intel_dst, INTEL_WRITE_PART);
-   src_bo = intel_bufferobj_source(intel, intel_src, 64, &src_offset);
-
-   intel_emit_linear_blit(intel,
+   intel_emit_linear_blit(brw,
 			  dst_bo, write_offset,
 			  src_bo, read_offset + src_offset, size);
 
@@ -798,14 +587,11 @@ intel_bufferobj_copy_subdata(struct gl_context *ctx,
     * flush.  Once again, we wish for a domain tracker in libdrm to cover
     * usage inside of a batchbuffer.
     */
-   intel_batchbuffer_emit_mi_flush(intel);
+   intel_batchbuffer_emit_mi_flush(brw);
 }
 
-#if FEATURE_APPLE_object_purgeable
 static GLenum
-intel_buffer_purgeable(struct gl_context * ctx,
-                       drm_intel_bo *buffer,
-                       GLenum option)
+intel_buffer_purgeable(drm_intel_bo *buffer)
 {
    int retained = 0;
 
@@ -820,25 +606,19 @@ intel_buffer_object_purgeable(struct gl_context * ctx,
                               struct gl_buffer_object *obj,
                               GLenum option)
 {
-   struct intel_buffer_object *intel;
+   struct intel_buffer_object *intel_obj = intel_buffer_object (obj);
 
-   intel = intel_buffer_object (obj);
-   if (intel->buffer != NULL)
-      return intel_buffer_purgeable (ctx, intel->buffer, option);
+   if (intel_obj->buffer != NULL)
+      return intel_buffer_purgeable(intel_obj->buffer);
 
    if (option == GL_RELEASED_APPLE) {
-      if (intel->sys_buffer != NULL) {
-         free(intel->sys_buffer);
-         intel->sys_buffer = NULL;
-      }
-
       return GL_RELEASED_APPLE;
    } else {
       /* XXX Create the buffer and madvise(MADV_DONTNEED)? */
-      return intel_buffer_purgeable (ctx,
-                                     intel_bufferobj_buffer(intel_context(ctx),
-                                                            intel, INTEL_READ),
-                                     option);
+      struct brw_context *brw = brw_context(ctx);
+      drm_intel_bo *bo = intel_bufferobj_buffer(brw, intel_obj, INTEL_READ);
+
+      return intel_buffer_purgeable(bo);
    }
 }
 
@@ -849,11 +629,14 @@ intel_texture_object_purgeable(struct gl_context * ctx,
 {
    struct intel_texture_object *intel;
 
+   (void) ctx;
+   (void) option;
+
    intel = intel_texture_object(obj);
    if (intel->mt == NULL || intel->mt->region == NULL)
       return GL_RELEASED_APPLE;
 
-   return intel_buffer_purgeable (ctx, intel->mt->region->buffer, option);
+   return intel_buffer_purgeable(intel->mt->region->bo);
 }
 
 static GLenum
@@ -863,17 +646,18 @@ intel_render_object_purgeable(struct gl_context * ctx,
 {
    struct intel_renderbuffer *intel;
 
+   (void) ctx;
+   (void) option;
+
    intel = intel_renderbuffer(obj);
-   if (intel->region == NULL)
+   if (intel->mt == NULL)
       return GL_RELEASED_APPLE;
 
-   return intel_buffer_purgeable (ctx, intel->region->buffer, option);
+   return intel_buffer_purgeable(intel->mt->region->bo);
 }
 
 static GLenum
-intel_buffer_unpurgeable(struct gl_context * ctx,
-                         drm_intel_bo *buffer,
-                         GLenum option)
+intel_buffer_unpurgeable(drm_intel_bo *buffer)
 {
    int retained;
 
@@ -889,7 +673,10 @@ intel_buffer_object_unpurgeable(struct gl_context * ctx,
                                 struct gl_buffer_object *obj,
                                 GLenum option)
 {
-   return intel_buffer_unpurgeable (ctx, intel_buffer_object (obj)->buffer, option);
+   (void) ctx;
+   (void) option;
+
+   return intel_buffer_unpurgeable(intel_buffer_object (obj)->buffer);
 }
 
 static GLenum
@@ -899,11 +686,14 @@ intel_texture_object_unpurgeable(struct gl_context * ctx,
 {
    struct intel_texture_object *intel;
 
+   (void) ctx;
+   (void) option;
+
    intel = intel_texture_object(obj);
    if (intel->mt == NULL || intel->mt->region == NULL)
       return GL_UNDEFINED_APPLE;
 
-   return intel_buffer_unpurgeable (ctx, intel->mt->region->buffer, option);
+   return intel_buffer_unpurgeable(intel->mt->region->bo);
 }
 
 static GLenum
@@ -913,13 +703,15 @@ intel_render_object_unpurgeable(struct gl_context * ctx,
 {
    struct intel_renderbuffer *intel;
 
+   (void) ctx;
+   (void) option;
+
    intel = intel_renderbuffer(obj);
-   if (intel->region == NULL)
+   if (intel->mt == NULL)
       return GL_UNDEFINED_APPLE;
 
-   return intel_buffer_unpurgeable (ctx, intel->region->buffer, option);
+   return intel_buffer_unpurgeable(intel->mt->region->bo);
 }
-#endif
 
 void
 intelInitBufferObjectFuncs(struct dd_function_table *functions)
@@ -929,13 +721,11 @@ intelInitBufferObjectFuncs(struct dd_function_table *functions)
    functions->BufferData = intel_bufferobj_data;
    functions->BufferSubData = intel_bufferobj_subdata;
    functions->GetBufferSubData = intel_bufferobj_get_subdata;
-   functions->MapBuffer = intel_bufferobj_map;
    functions->MapBufferRange = intel_bufferobj_map_range;
    functions->FlushMappedBufferRange = intel_bufferobj_flush_mapped_range;
    functions->UnmapBuffer = intel_bufferobj_unmap;
    functions->CopyBufferSubData = intel_bufferobj_copy_subdata;
 
-#if FEATURE_APPLE_object_purgeable
    functions->BufferObjectPurgeable = intel_buffer_object_purgeable;
    functions->TextureObjectPurgeable = intel_texture_object_purgeable;
    functions->RenderObjectPurgeable = intel_render_object_purgeable;
@@ -943,5 +733,4 @@ intelInitBufferObjectFuncs(struct dd_function_table *functions)
    functions->BufferObjectUnpurgeable = intel_buffer_object_unpurgeable;
    functions->TextureObjectUnpurgeable = intel_texture_object_unpurgeable;
    functions->RenderObjectUnpurgeable = intel_render_object_unpurgeable;
-#endif
 }
