@@ -1,6 +1,5 @@
 /*
  * Mesa 3-D graphics library
- * Version:  7.5
  *
  * Copyright (C) 1999-2008  Brian Paul   All Rights Reserved.
  * Copyright (C) 2009  VMware, Inc.  All Rights Reserved.
@@ -18,9 +17,10 @@
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
- * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
+ * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+ * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
  */
 
 /** 
@@ -31,13 +31,13 @@
 
 #include <stdbool.h>
 #include "main/glheader.h"
+#include "main/blend.h"
 #include "main/colormac.h"
 #include "main/context.h"
 #include "main/enums.h"
 #include "main/formats.h"
 #include "main/glformats.h"
 #include "main/macros.h"
-#include "main/mfeatures.h"
 #include "main/mtypes.h"
 #include "main/state.h"
 #include "main/texcompress.h"
@@ -175,6 +175,16 @@ get_texobj(struct gl_context *ctx, GLenum target, GLboolean get)
          return texUnit->CurrentTex[TEXTURE_CUBE_ARRAY_INDEX];
       }
       break;
+   case GL_TEXTURE_2D_MULTISAMPLE:
+      if (ctx->Extensions.ARB_texture_multisample) {
+         return texUnit->CurrentTex[TEXTURE_2D_MULTISAMPLE_INDEX];
+      }
+      break;
+   case GL_TEXTURE_2D_MULTISAMPLE_ARRAY:
+      if (ctx->Extensions.ARB_texture_multisample) {
+         return texUnit->CurrentTex[TEXTURE_2D_MULTISAMPLE_ARRAY_INDEX];
+      }
+      break;
    default:
       ;
    }
@@ -250,6 +260,20 @@ incomplete(struct gl_context *ctx, struct gl_texture_object *texObj)
 }
 
 
+static GLboolean
+target_allows_setting_sampler_parameters(GLenum target)
+{
+   switch (target) {
+   case GL_TEXTURE_2D_MULTISAMPLE:
+   case GL_TEXTURE_2D_MULTISAMPLE_ARRAY:
+      return GL_FALSE;
+
+   default:
+      return GL_TRUE;
+   }
+}
+
+
 /**
  * Set an integer-valued texture parameter
  * \return GL_TRUE if legal AND the value changed, GL_FALSE otherwise
@@ -261,6 +285,9 @@ set_tex_parameteri(struct gl_context *ctx,
 {
    switch (pname) {
    case GL_TEXTURE_MIN_FILTER:
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.MinFilter == params[0])
          return GL_FALSE;
       switch (params[0]) {
@@ -286,6 +313,9 @@ set_tex_parameteri(struct gl_context *ctx,
       return GL_FALSE;
 
    case GL_TEXTURE_MAG_FILTER:
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.MagFilter == params[0])
          return GL_FALSE;
       switch (params[0]) {
@@ -300,6 +330,9 @@ set_tex_parameteri(struct gl_context *ctx,
       return GL_FALSE;
 
    case GL_TEXTURE_WRAP_S:
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.WrapS == params[0])
          return GL_FALSE;
       if (validate_texture_wrap_mode(ctx, texObj->Target, params[0])) {
@@ -310,6 +343,9 @@ set_tex_parameteri(struct gl_context *ctx,
       return GL_FALSE;
 
    case GL_TEXTURE_WRAP_T:
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.WrapT == params[0])
          return GL_FALSE;
       if (validate_texture_wrap_mode(ctx, texObj->Target, params[0])) {
@@ -320,6 +356,9 @@ set_tex_parameteri(struct gl_context *ctx,
       return GL_FALSE;
 
    case GL_TEXTURE_WRAP_R:
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.WrapR == params[0])
          return GL_FALSE;
       if (validate_texture_wrap_mode(ctx, texObj->Target, params[0])) {
@@ -335,6 +374,11 @@ set_tex_parameteri(struct gl_context *ctx,
 
       if (texObj->BaseLevel == params[0])
          return GL_FALSE;
+
+      if ((texObj->Target == GL_TEXTURE_2D_MULTISAMPLE ||
+           texObj->Target == GL_TEXTURE_2D_MULTISAMPLE_ARRAY) && params[0] != 0)
+         goto invalid_operation;
+
       if (params[0] < 0 ||
           (texObj->Target == GL_TEXTURE_RECTANGLE_ARB && params[0] != 0)) {
          _mesa_error(ctx, GL_INVALID_VALUE,
@@ -348,6 +392,7 @@ set_tex_parameteri(struct gl_context *ctx,
    case GL_TEXTURE_MAX_LEVEL:
       if (texObj->MaxLevel == params[0])
          return GL_FALSE;
+
       if (params[0] < 0 || texObj->Target == GL_TEXTURE_RECTANGLE_ARB) {
          _mesa_error(ctx, GL_INVALID_VALUE,
                      "glTexParameter(param=%d)", params[0]);
@@ -373,6 +418,10 @@ set_tex_parameteri(struct gl_context *ctx,
    case GL_TEXTURE_COMPARE_MODE_ARB:
       if ((_mesa_is_desktop_gl(ctx) && ctx->Extensions.ARB_shadow)
           || _mesa_is_gles3(ctx)) {
+
+         if (!target_allows_setting_sampler_parameters(texObj->Target))
+            goto invalid_operation;
+
          if (texObj->Sampler.CompareMode == params[0])
             return GL_FALSE;
          if (params[0] == GL_NONE ||
@@ -388,26 +437,24 @@ set_tex_parameteri(struct gl_context *ctx,
    case GL_TEXTURE_COMPARE_FUNC_ARB:
       if ((_mesa_is_desktop_gl(ctx) && ctx->Extensions.ARB_shadow)
           || _mesa_is_gles3(ctx)) {
+
+         if (!target_allows_setting_sampler_parameters(texObj->Target))
+            goto invalid_operation;
+
          if (texObj->Sampler.CompareFunc == params[0])
             return GL_FALSE;
          switch (params[0]) {
          case GL_LEQUAL:
          case GL_GEQUAL:
-            flush(ctx);
-            texObj->Sampler.CompareFunc = params[0];
-            return GL_TRUE;
          case GL_EQUAL:
          case GL_NOTEQUAL:
          case GL_LESS:
          case GL_GREATER:
          case GL_ALWAYS:
          case GL_NEVER:
-            if (ctx->Extensions.EXT_shadow_funcs) {
-               flush(ctx);
-               texObj->Sampler.CompareFunc = params[0];
-               return GL_TRUE;
-            }
-            /* fall-through */
+            flush(ctx);
+            texObj->Sampler.CompareFunc = params[0];
+            return GL_TRUE;
          default:
             goto invalid_param;
          }
@@ -489,7 +536,11 @@ set_tex_parameteri(struct gl_context *ctx,
    case GL_TEXTURE_SRGB_DECODE_EXT:
       if (_mesa_is_desktop_gl(ctx)
           && ctx->Extensions.EXT_texture_sRGB_decode) {
-	 GLenum decode = params[0];
+         GLenum decode = params[0];
+
+         if (!target_allows_setting_sampler_parameters(texObj->Target))
+            goto invalid_operation;
+
 	 if (decode == GL_DECODE_EXT || decode == GL_SKIP_DECODE_EXT) {
 	    if (texObj->Sampler.sRGBDecode != decode) {
 	       flush(ctx);
@@ -504,6 +555,10 @@ set_tex_parameteri(struct gl_context *ctx,
       if (_mesa_is_desktop_gl(ctx)
           && ctx->Extensions.AMD_seamless_cubemap_per_texture) {
          GLenum param = params[0];
+
+         if (!target_allows_setting_sampler_parameters(texObj->Target))
+            goto invalid_operation;
+
          if (param != GL_TRUE && param != GL_FALSE) {
             goto invalid_param;
          }
@@ -528,6 +583,11 @@ invalid_param:
    _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(param=%s)",
                _mesa_lookup_enum_by_nr(params[0]));
    return GL_FALSE;
+
+invalid_operation:
+   _mesa_error(ctx, GL_INVALID_OPERATION, "glTexParameter(pname=%s)",
+               _mesa_lookup_enum_by_nr(pname));
+   return GL_FALSE;
 }
 
 
@@ -545,6 +605,9 @@ set_tex_parameterf(struct gl_context *ctx,
       if (!_mesa_is_desktop_gl(ctx) && !_mesa_is_gles3(ctx))
          goto invalid_pname;
 
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.MinLod == params[0])
          return GL_FALSE;
       flush(ctx);
@@ -555,6 +618,9 @@ set_tex_parameterf(struct gl_context *ctx,
       if (!_mesa_is_desktop_gl(ctx) && !_mesa_is_gles3(ctx))
          goto invalid_pname;
 
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.MaxLod == params[0])
          return GL_FALSE;
       flush(ctx);
@@ -571,6 +637,9 @@ set_tex_parameterf(struct gl_context *ctx,
 
    case GL_TEXTURE_MAX_ANISOTROPY_EXT:
       if (ctx->Extensions.EXT_texture_filter_anisotropic) {
+         if (!target_allows_setting_sampler_parameters(texObj->Target))
+            goto invalid_operation;
+
          if (texObj->Sampler.MaxAnisotropy == params[0])
             return GL_FALSE;
          if (params[0] < 1.0) {
@@ -591,13 +660,13 @@ set_tex_parameterf(struct gl_context *ctx,
       return GL_FALSE;
 
    case GL_TEXTURE_LOD_BIAS:
-      /* NOTE: this is really part of OpenGL 1.4, not EXT_texture_lod_bias.
-       * It was removed in core-profile, and it has never existed in OpenGL
-       * ES.
-       */
-      if (ctx->API != API_OPENGL_COMPAT)
+      /* NOTE: this is really part of OpenGL 1.4, not EXT_texture_lod_bias. */
+      if (_mesa_is_gles(ctx))
          goto invalid_pname;
 
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       if (texObj->Sampler.LodBias != params[0]) {
 	 flush(ctx);
 	 texObj->Sampler.LodBias = params[0];
@@ -609,6 +678,9 @@ set_tex_parameterf(struct gl_context *ctx,
       if (!_mesa_is_desktop_gl(ctx))
          goto invalid_pname;
 
+      if (!target_allows_setting_sampler_parameters(texObj->Target))
+         goto invalid_operation;
+
       flush(ctx);
       /* ARB_texture_float disables clamping */
       if (ctx->Extensions.ARB_texture_float) {
@@ -633,6 +705,11 @@ invalid_pname:
    _mesa_error(ctx, GL_INVALID_ENUM, "glTexParameter(pname=%s)",
                _mesa_lookup_enum_by_nr(pname));
    return GL_FALSE;
+
+invalid_operation:
+   _mesa_error(ctx, GL_INVALID_OPERATION, "glTexParameter(pname=%s)",
+               _mesa_lookup_enum_by_nr(pname));
+   return GL_FALSE;
 }
 
 
@@ -966,6 +1043,11 @@ legal_get_tex_level_parameter_target(struct gl_context *ctx, GLenum target)
        * "target may also be TEXTURE_BUFFER, indicating the texture buffer."
        */
       return ctx->API == API_OPENGL_CORE && ctx->Version >= 31;
+   case GL_TEXTURE_2D_MULTISAMPLE:
+   case GL_TEXTURE_2D_MULTISAMPLE_ARRAY:
+   case GL_PROXY_TEXTURE_2D_MULTISAMPLE:
+   case GL_PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY:
+      return ctx->Extensions.ARB_texture_multisample;
    default:
       return GL_FALSE;
    }
@@ -1105,6 +1187,19 @@ get_tex_level_parameter_image(struct gl_context *ctx,
 	    *params = GL_NONE;
          break;
 
+      /* GL_ARB_texture_multisample */
+      case GL_TEXTURE_SAMPLES:
+         if (!ctx->Extensions.ARB_texture_multisample)
+            goto invalid_pname;
+         *params = img->NumSamples;
+         break;
+
+      case GL_TEXTURE_FIXED_SAMPLE_LOCATIONS:
+         if (!ctx->Extensions.ARB_texture_multisample)
+            goto invalid_pname;
+         *params = img->FixedSampleLocations;
+         break;
+
       default:
          goto invalid_pname;
    }
@@ -1181,6 +1276,18 @@ get_tex_level_parameter_buffer(struct gl_context *ctx,
          *params = _mesa_get_format_bits(texFormat, pname);
          break;
 
+      /* GL_ARB_texture_buffer_range */
+      case GL_TEXTURE_BUFFER_OFFSET:
+         if (!ctx->Extensions.ARB_texture_buffer_range)
+            goto invalid_pname;
+         *params = texObj->BufferOffset;
+         break;
+      case GL_TEXTURE_BUFFER_SIZE:
+         if (!ctx->Extensions.ARB_texture_buffer_range)
+            goto invalid_pname;
+         *params = (texObj->BufferSize == -1) ? bo->Size : texObj->BufferSize;
+         break;
+
       /* GL_ARB_texture_compression */
       case GL_TEXTURE_COMPRESSED_IMAGE_SIZE:
          /* Always illegal for GL_TEXTURE_BUFFER */
@@ -1301,7 +1408,7 @@ _mesa_GetTexParameterfv( GLenum target, GLenum pname, GLfloat *params )
 
          if (ctx->NewState & (_NEW_BUFFERS | _NEW_FRAG_CLAMP))
             _mesa_update_state_locked(ctx);
-         if (ctx->Color._ClampFragmentColor) {
+         if (_mesa_get_clamp_fragment_color(ctx)) {
             params[0] = CLAMP(obj->Sampler.BorderColor.f[0], 0.0F, 1.0F);
             params[1] = CLAMP(obj->Sampler.BorderColor.f[1], 0.0F, 1.0F);
             params[2] = CLAMP(obj->Sampler.BorderColor.f[2], 0.0F, 1.0F);
@@ -1379,7 +1486,7 @@ _mesa_GetTexParameterfv( GLenum target, GLenum pname, GLfloat *params )
          *params = (GLfloat) obj->DepthMode;
          break;
       case GL_TEXTURE_LOD_BIAS:
-         if (ctx->API != API_OPENGL_COMPAT)
+         if (_mesa_is_gles(ctx))
             goto invalid_pname;
 
          *params = obj->Sampler.LodBias;
@@ -1427,11 +1534,21 @@ _mesa_GetTexParameterfv( GLenum target, GLenum pname, GLfloat *params )
          break;
 
       case GL_TEXTURE_IMMUTABLE_FORMAT:
-         if (!ctx->Extensions.ARB_texture_storage)
-            goto invalid_pname;
          *params = (GLfloat) obj->Immutable;
          break;
 
+      case GL_TEXTURE_IMMUTABLE_LEVELS:
+         if (!_mesa_is_gles3(ctx))
+            goto invalid_pname;
+         *params = (GLfloat) obj->ImmutableLevels;
+         break;
+
+      case GL_REQUIRED_TEXTURE_IMAGE_UNITS_OES:
+         if (!_mesa_is_gles(ctx) || !ctx->Extensions.OES_EGL_image_external)
+            goto invalid_pname;
+         *params = (GLfloat) obj->RequiredTextureImageUnits;
+         break;
+
       case GL_TEXTURE_SRGB_DECODE_EXT:
          if (!ctx->Extensions.EXT_texture_sRGB_decode)
             goto invalid_pname;
@@ -1557,10 +1674,13 @@ _mesa_GetTexParameteriv( GLenum target, GLenum pname, GLint *params )
          *params = (GLint) obj->DepthMode;
          break;
       case GL_TEXTURE_LOD_BIAS:
-         if (ctx->API != API_OPENGL_COMPAT)
+         if (_mesa_is_gles(ctx))
             goto invalid_pname;
 
-         *params = (GLint) obj->Sampler.LodBias;
+         /* GL spec 'Data Conversions' section specifies that floating-point
+          * value in integer Get function is rounded to nearest integer
+          */
+         *params = (GLint) roundf(obj->Sampler.LodBias);
          break;
       case GL_TEXTURE_CROP_RECT_OES:
          if (ctx->API != API_OPENGLES || !ctx->Extensions.OES_draw_texture)
@@ -1598,11 +1718,15 @@ _mesa_GetTexParameteriv( GLenum target, GLenum pname, GLint *params )
          break;
 
       case GL_TEXTURE_IMMUTABLE_FORMAT:
-         if (!ctx->Extensions.ARB_texture_storage)
-            goto invalid_pname;
          *params = (GLint) obj->Immutable;
          break;
 
+      case GL_TEXTURE_IMMUTABLE_LEVELS:
+         if (!_mesa_is_gles3(ctx))
+            goto invalid_pname;
+         *params = obj->ImmutableLevels;
+         break;
+
       case GL_REQUIRED_TEXTURE_IMAGE_UNITS_OES:
          if (!_mesa_is_gles(ctx) || !ctx->Extensions.OES_EGL_image_external)
             goto invalid_pname;
